@@ -74,7 +74,8 @@ int main(int argc, char **argv){
 	log_mesg(0, 1, 1, debug, "The Image magic error. This file is NOT partclone Image\n");
 
     /// alloc a memory to restore bitmap
-    bitmap = (unsigned long*)malloc(sizeof(unsigned long)*LONGS(image_hdr.totalblock));
+    //bitmap = (unsigned long*)malloc(sizeof(unsigned long)*LONGS(image_hdr.totalblock));
+    bitmap = (unsigned long*)calloc(sizeof(unsigned long), sizeof(unsigned long)*LONGS(image_hdr.totalblock));
 
     log_mesg(0, 0, 0, debug, "initial main bitmap pointer %lli\n", bitmap);
     log_mesg(0, 0, 0, debug, "Initial image hdr: read bitmap table\n");
