@@ -224,9 +224,7 @@ extern void Ncurses_progress_update(struct progress_bar *prog, unsigned long lon
 {
 #ifdef HAVE_LIBNCURSESW
 
-    char *clear_buf = NULL;
-    char r_block[] = "                                                  ";
-    char p_block[50];
+    char *p_block;
     prog_stat_t prog_stat;
 
     memset(&prog_stat, 0, sizeof(prog_stat_t));
@@ -249,20 +247,12 @@ extern void Ncurses_progress_update(struct progress_bar *prog, unsigned long lon
         mvwprintw(p_win, 2, 0, _("Remaining: %s"), prog_stat.Rformated);
 	if ((int)prog_stat.speed > 0)
 	    mvwprintw(p_win, 3, 0, _("Rate: %6.2f%s/min"), (float)(prog_stat.speed), prog_stat.speed_unit);
-        //mvwprintw(p_win, 3, 0, _("Completed:%6.2f%%"), percent);
+        p_block = calloc(sizeof(char), 50);
+        memset(p_block, ' ', (size_t)(prog_stat.percent*0.5));
         wattrset(bar_win, COLOR_PAIR(4));
-        strncpy(p_block, r_block, (prog_stat.percent/2)-1);
         mvwprintw(bar_win, 0, 0, "%s", p_block);
         wattroff(bar_win, COLOR_PAIR(4));
-        if(prog_stat.percent <= 50){
-            wattrset(p_win, COLOR_PAIR(5));
-            mvwprintw(p_win, 5, 25, "%3.0f%%", prog_stat.percent);
-            wattroff(p_win, COLOR_PAIR(5));
-        }else{
-            wattrset(p_win, COLOR_PAIR(6));
-            mvwprintw(p_win, 5, 25, "%3.0f%%", prog_stat.percent);
-            wattroff(p_win, COLOR_PAIR(6));
-        }
+	free(p_block);
         mvwprintw(p_win, 5, 52, "%6.2f%%", prog_stat.percent);
         wrefresh(p_win);
         wrefresh(bar_win);
@@ -273,11 +263,9 @@ extern void Ncurses_progress_update(struct progress_bar *prog, unsigned long lon
 	    mvwprintw(p_win, 3, 0, _("Ave. Rate: %6.2f%s/min"), (float)prog_stat.speed, prog_stat.speed_unit);
 
         wattrset(bar_win, COLOR_PAIR(4));
-        mvwprintw(bar_win, 0, 0, "%s", r_block);
+	mvwprintw(bar_win, 0, 0, "%50s", " ");
         wattroff(bar_win, COLOR_PAIR(4));
-        wattrset(p_win, COLOR_PAIR(6));
-        mvwprintw(p_win, 5, 22, "%6.2f%%", prog_stat.percent);
-        wattroff(p_win, COLOR_PAIR(6));
+
         mvwprintw(p_win, 5, 52, "%6.2f%%", prog_stat.percent);
         wrefresh(p_win);
         wrefresh(bar_win);
